/*
	JELAIR-HotAirBalloon(UCGOandUMmu)
	Made by JELAIR
	built upon ShuttlePB.cpp from ORBITER SDK by Martin Schweiger
	Date: 20110211
	----------------------------------------------

	Credits go to:

		CPP Code and theory:
			computerex http://orbiter-forum.com/member.php?u=12
			Hielor http://orbiter-forum.com/member.php?u=916
			BrianJ http://orbiter-forum.com/member.php?u=615
			atomicdryad http://orbiter-forum.com/member.php?u=5863

	----------------------------------------------

	Freeware for non-commercial use.
	JEL
*/


// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "UmmuSDK.h"
#include "UCGOCargoSDK.h"
#include "OrbiterSoundSDK35.h"
#define BURNER				1
#define VentFlap				2


// ==============================================================
// Some vessel parameters
// ==============================================================
const double  PB_SIZE       = 17.75;             // mean radius [m]
	VECTOR3 PB_CS         = {255.61,246.24,255.61}; // x,y,z cross sections [m^2]
const VECTOR3 PB_PMI        = {109.2,29.36,109.2};// principal moments of inertia (mass-normalised) [m^2]
const VECTOR3 PB_RD         = {0.025,0.025,0.025};// rotation drag coefficients
const double  PB_EMPTYMASS  = 199.6; //539.8;   // empty vessel mass [kg] / including envelope (2831.7 m^3), basket (5 pax), double burner, WITHOUT passengers (5 pax)
//const double  PB_EMPTYMASS  = 539.8;   // empty vessel mass [kg] / including envelope (2831.7 m^3), basket (5 pax), double burner, WITH passengers (5 pax)
const double  PB_FUELMASS   = 183.7;           // max fuel mass [kg], 3 20-gallon (75.7-liter) fuel tanks full of propane
const double  PB_ISP        = 5e4;             // fuel-specific impulse [m/s]
const VECTOR3 PB_TDP[3]     = {{0,-17,1.5},{-1.5,-17,-1.5},{1.5,-17,-1.5}}; // touchdown points [m]
const VECTOR3 PB_COP        = {0,0,0};		   // centre of pressure for airfoils [m]
const double  PB_VLIFT_C    = 0.0;             // chord length [m]
const double  PB_VLIFT_S    = 0.0;             // wing area [m^2]
const double  PB_VLIFT_A    = 0.0;             // wing aspect ratio
const double  PB_HLIFT_C    = 0.0;             // chord length [m]
const double  PB_HLIFT_S    = 0.0;             // wing area [m^2]
const double  PB_HLIFT_A    = 0.0;             // wing aspect ratio

const double  PB_MAXMAINTH  = 3e4;             
const double  PB_MAXHOVERTH = 1.5e4;
const double  PB_MAXRCSTH   = 2e2;

const VECTOR3 PB_DOCK_POS   = {0,1.3,-1};      // docking port location [m]
const VECTOR3 PB_DOCK_DIR   = {0,1,0};         // docking port approach direction
const VECTOR3 PB_DOCK_ROT   = {0,0,-1};        // docking port alignment direction



	//JELAIR
	int Control = 0; //JELAIR, is the burner ON or OFF
	int ControlAutopilot = 0; //JELAIR, is the autopilot ON or OFF
	int EnvelopeAirVentControl = 0; //JELAIR, is the envelope air-vent open or closed (to let our hot air for landing and quick descents)
	double F = 0; //JELAIR, amount of Newton force added (this is how the balloon flies)

	double lift_generated_by_the_envelope = 0; //JELAIR, kg
	double envelope_volume = 2831.7; //JELAIR, m^3 (cubic meter)
	double ambient_air_pressure_outside_envelope = 0; //JELAIR, the ambient air pressure of the surrounding air
//	double air_temperature_outside_the_envelope = 0; //JELAIR, the ambient air temperature of the surrounding air
	double air_temperature_inside_the_envelope = 0; //JELAIR, the air temperature of the air inside the envelope

	double air_density_inside_the_envelope = 0; //JELAIR
	double air_mass_inside_the_envelope = 0; //JELAIR

	int AutopilotAltitudeHold;
	int is_envelope_overheated_and_melted;
	//JELAIR



// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
double LiftCoeff (double aoa)
{
	int i;
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-60*RAD,-30*RAD,-1*RAD,15*RAD,20*RAD,25*RAD,60*RAD,180*RAD};
	static const double CL[nlift]  = {       0,      0,      0,     0,     0,     0,     0,     0,      0};
	static const double SCL[nlift] = {(CL[1]-CL[0])/(AOA[1]-AOA[0]), (CL[2]-CL[1])/(AOA[2]-AOA[1]),
		                              (CL[3]-CL[2])/(AOA[3]-AOA[2]), (CL[4]-CL[3])/(AOA[4]-AOA[3]),
									  (CL[5]-CL[4])/(AOA[5]-AOA[4]), (CL[6]-CL[5])/(AOA[6]-AOA[5]),
									  (CL[7]-CL[6])/(AOA[7]-AOA[6]), (CL[8]-CL[7])/(AOA[8]-AOA[7])};
	for (i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	return CL[i] + (aoa-AOA[i])*SCL[i];
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class JELAIR2011HotAirBalloon: public VESSEL3 {
public:
	JELAIR2011HotAirBalloon (OBJHANDLE hVessel, int flightmodel);
	~JELAIR2011HotAirBalloon ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkSaveState(FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	void clbkPreStep (double simt, double simdt, double mjd);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkPostCreation(void);
	int JELAIR_HAB20110208;	// this is your unique Id, it will identify your ship in OrbiterSound.
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkVisualCreated (VISHANDLE vis, int refcount);

	// UMMU 2.0 DECLARATION
	UMMUCREWMANAGMENT Crew;
	int SelectedUmmuMember;				// for the SDK demo, select the member to eva
//	int iActionAreaDemoStep;			// this is just to show one feature of action area.
	void clbkSetClassCaps_UMMu(void);	// our special SetClassCap function just added for more readability

	// The HUD display method variable, see PDF doc
	char cUmmuHudDisplay[255];			// UMmu hud char variable
	double dHudMessageDelay;			// UMmu hud display delay
	char *SendHudMessage(void);			// UMmu hud display function

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	char cAddUMmuToVessel[255];
	void AddUMmuToVessel(BOOL bStartAdding=FALSE);

	// UCGO SPECIFIC CODE OF CLASS HEADER **********************************
	void clbkSetClassCapsUCgo (void); // a custom UCGO clbkSetClassCap just to keep code clear

	// UCGO 2.0 CLASS HANDLE FUNCTION AND VARIABLES
	UCGO	hUcgo;						// Cargo class handle
	char   *SendCargHudMessage(void);	// Cargo hud display function
	char	cCargoHudDisplay[255];		// Cargo hud display char variable
	double	dCargHudMessageDelay;		// Cargo hud display delay
	int		iSelectedCargo;				// for the selection of cargos -1 by default

	
	
private:
	static void vlift (VESSEL *v, double aoa, double M, double Re,
		void *context, double *cl, double *cm, double *cd);
	static void hlift (VESSEL *v, double aoa, double M, double Re,
		void *context, double *cl, double *cm, double *cd);

	// transformations for control surface animations
//	static MGROUP_ROTATE trans_Laileron, trans_Raileron;
//	static MGROUP_ROTATE trans_Lelevator, trans_Relevator;
};

JELAIR2011HotAirBalloon::JELAIR2011HotAirBalloon (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
}

JELAIR2011HotAirBalloon::~JELAIR2011HotAirBalloon ()
{
}

// animation transformation definitions
static UINT GRP_LWING = 2;
static UINT GRP_RWING = 3;
static VECTOR3 LWING_REF  = {-1.3,-0.725,-1.5};
static VECTOR3 LWING_AXIS = {-0.9619,-0.2735,0};
static VECTOR3 RWING_REF  = {1.3,-0.725,-1.5};
static VECTOR3 RWING_AXIS = {0.9619,-0.2735,0};
static float AILERON_RANGE = (float)(20.0*RAD);
static float ELEVATOR_RANGE = (float)(30.0*RAD);
//MGROUP_ROTATE JELAIR2011HotAirBalloon::trans_Laileron (0, &GRP_LWING, 1, LWING_REF, LWING_AXIS, AILERON_RANGE);
//MGROUP_ROTATE JELAIR2011HotAirBalloon::trans_Raileron (0, &GRP_RWING, 1, RWING_REF, RWING_AXIS, AILERON_RANGE);
//MGROUP_ROTATE JELAIR2011HotAirBalloon::trans_Lelevator (0, &GRP_LWING, 1, LWING_REF, LWING_AXIS, -ELEVATOR_RANGE);
//MGROUP_ROTATE JELAIR2011HotAirBalloon::trans_Relevator (0, &GRP_RWING, 1, RWING_REF, RWING_AXIS, ELEVATOR_RANGE);


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void JELAIR2011HotAirBalloon::clbkSetClassCaps (FILEHANDLE cfg)
{
	THRUSTER_HANDLE th_main, th_hover, th_rcs[14], th_group[4];

	// physical vessel parameters
	SetSize (PB_SIZE);
	SetEmptyMass (PB_EMPTYMASS);
	SetPMI (PB_PMI);
	SetCrossSections (PB_CS);
	SetRotDrag (PB_RD);
	SetTouchdownPoints (PB_TDP[0], PB_TDP[1], PB_TDP[2]);

	// docking port definitions
	SetDockParams (PB_DOCK_POS, PB_DOCK_DIR, PB_DOCK_ROT);

	// airfoil definitions
	CreateAirfoil3 (LIFT_VERTICAL,   PB_COP, vlift, NULL, PB_VLIFT_C, PB_VLIFT_S, PB_VLIFT_A);
	CreateAirfoil3 (LIFT_HORIZONTAL, PB_COP, hlift, NULL, PB_HLIFT_C, PB_HLIFT_S, PB_HLIFT_A);

	// control surface animations
//	UINT anim_Laileron = CreateAnimation (0.5);
//	UINT anim_Raileron = CreateAnimation (0.5);
//	UINT anim_elevator = CreateAnimation (0.5);
//	AddAnimationComponent (anim_Laileron, 0, 1, &trans_Laileron);
//	AddAnimationComponent (anim_Raileron, 0, 1, &trans_Raileron);
//	AddAnimationComponent (anim_elevator, 0, 1, &trans_Lelevator);
//	AddAnimationComponent (anim_elevator, 0, 1, &trans_Relevator);

	// aerodynamic control surface defintions
	CreateControlSurface (AIRCTRL_ELEVATOR, 1.5, 0.7, _V( 0,0,-2.5), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON, 1.5, 0.25, _V( 1,0,-2.5), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON, 1.5, 0.25, _V(-1,0,-2.5), AIRCTRL_AXIS_XNEG);

	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);

	// main engine
	th_main = CreateThruster (_V(0,0,-4.35), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	AddExhaust (th_main, 8, 1, _V(0,0.3,-4.35), _V(0,0,-1));

	PARTICLESTREAMSPEC contrail_main = {
		0, 5.0, 16, 200, 0.15, 1.0, 5, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	PARTICLESTREAMSPEC exhaust_main = {
		0, 2.0, 20, 200, 0.05, 0.1, 8, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	AddExhaustStream (th_main, _V(0,0.3,-10), &contrail_main);
	AddExhaustStream (th_main, _V(0,0.3,-5), &exhaust_main);

	// hover engine
	th_hover = CreateThruster (_V(0,-1.5,0), _V(0,1,0), PB_MAXHOVERTH, hpr, PB_ISP);
	CreateThrusterGroup (&th_hover, 1, THGROUP_HOVER);
	AddExhaust (th_hover, 8, 1, _V(0,-1.5,1), _V(0,-1,0));
	AddExhaust (th_hover, 8, 1, _V(0,-1.5,-1), _V(0,-1,0));

	PARTICLESTREAMSPEC contrail_hover = {
		0, 5.0, 8, 200, 0.15, 1.0, 5, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 2,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
	PARTICLESTREAMSPEC exhaust_hover = {
		0, 2.0, 10, 200, 0.05, 0.05, 8, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

	AddExhaustStream (th_hover, _V(0,-3, 1), &contrail_hover);
	AddExhaustStream (th_hover, _V(0,-3,-1), &contrail_hover);
	AddExhaustStream (th_hover, _V(0,-2, 1), &exhaust_hover);
	AddExhaustStream (th_hover, _V(0,-2,-1), &exhaust_hover);

	// RCS engines
	th_rcs[ 0] = CreateThruster (_V( 1,0, 3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 1] = CreateThruster (_V( 1,0, 3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 2] = CreateThruster (_V(-1,0, 3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 3] = CreateThruster (_V(-1,0, 3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 4] = CreateThruster (_V( 1,0,-3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 5] = CreateThruster (_V( 1,0,-3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 6] = CreateThruster (_V(-1,0,-3), _V(0, 1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 7] = CreateThruster (_V(-1,0,-3), _V(0,-1,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 8] = CreateThruster (_V( 1,0, 3), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[ 9] = CreateThruster (_V(-1,0, 3), _V( 1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[10] = CreateThruster (_V( 1,0,-3), _V(-1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[11] = CreateThruster (_V(-1,0,-3), _V( 1,0,0), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[12] = CreateThruster (_V( 0,0,-3), _V(0,0, 1), PB_MAXRCSTH, hpr, PB_ISP);
	th_rcs[13] = CreateThruster (_V( 0,0, 3), _V(0,0,-1), PB_MAXRCSTH, hpr, PB_ISP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);

	// camera parameters
	SetCameraOffset (_V(0,-15,0));

	// associate a mesh for the visual
	AddMesh ("JELAIR\\BalloonEnvelope_2831.7");
//	VECTOR3 MeshVector = {0, 18, 0};
//	ShiftMesh(0, MeshVector);

	// Ummu call of our own function to keep listing clear.
	SelectedUmmuMember=0;
	clbkSetClassCaps_UMMu();

	// our call to our custom UCGO clbkSetClassCap
	// the only purpose is to keep code clear
	clbkSetClassCapsUCgo();

}





///////////////////////////////////////////////////////////////////////////////////////////
// clbkSetClassCaps function of Orbiter !REQUIRED!
// ----------------
// The codes below SHOULD be put in "clbkSetClassCaps" Orbiter's function of your vessel
// I made here a separate function called by "clbkSetClassCaps" for more readability.
//
// initialisation of Ummu goes here
///////////////////////////////////////////////////////////////////////////////////////////
void JELAIR2011HotAirBalloon::clbkSetClassCaps_UMMu (void)
{
	//--------------------------------------------------------------
	// InitUmmu
	// initialisation of Ummu must be done first ! return of 1 mean OK 
	// return of -999 mean Ummu addon is not installed in user's Orbiter, return of -1 
	// mean misc error. If error simply all function will fail silently 
	// and there will be no MMu. It's a good idea to warn the user. 
	// (see function "WarnUserUMMUNotInstalled" below)
	Crew.InitUmmu(GetHandle());	
	//--------------------------------------------------------------

	//--------------------------------------------------------------
	// GetUserUMmuVersion
	// this check wich version of UMmu addon the user have in his orbiter directory.
	// this may be usefull for future version with more function so you can check
	// if user installation support the new functions. Return -1 on error (not installed, version cannot be read)
	// or version number in float value (ie: 2.0="2.0" version etc etc)
	float UMmuVersion=Crew.GetUserUMmuVersion();

	//--------------------------------------------------------------
	// DefineAirLockShape
	// We set the airlock shape and state, this virtual box will be the one
	// into were a Ummu asking to reenter will be taken in account
	// first parameter is airlock door state (OPEN=TRUE) and other are X,X1,Y,Y1 and Z,Z1 
	// min and max coordinate for this virtual box (vessel local coordinate)
	// Pay attention to reenter when landed (if suitable) and make the box large enough so 
	// it's not a pain for the user to find airlock's entry. (See PDF doc)
	Crew.DefineAirLockShape(TRUE,-2,2,-18,-14,-2,2);			// Airlock open, 2 meter large 4 meter high 2 meter long (vessel local coordinate)

	//--------------------------------------------------------------
	// SetMembersPositionOrientationOnEVA
	// We define here were the Ummu will appear when they make an EVA
	// first parameter is position second is orientation, take care not to make
	// them appear to high on ground (will fall and die) or to low (Orbiter "light speed" bug)
	Crew.SetMembersPosRotOnEVA(_V(0,-15.5,2),_V(0,0,0));	// 2 meters in front of ship (z) facing direction of ship (vessel local coordinate)

	//---------------------------------------------------------------
	// finally set the maximum seat available in this ship, default is
	// 8 but it can go from 1 to 100
	// BAD IDEA: using this to "close" your ship, use "SetAirlockDoorState" instead.
	Crew.SetMaxSeatAvailableInShip(5);

	//----------------------------------------------------------------
	// OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL
	// DeclareActionArea
	// Ever wanted to be able to open a door, repair a system or trigger something in your vessel from an UMMU ?
	// Here come a new feature of UMMU 2.0: Action area. See UMmuSDK.h and PDF doc for complete explanation and see
	// "DetectActionAreaActivated" below.
//	Crew.DeclareActionArea(0,_V(-5,0,0),2.5,TRUE,"action_activated.wav","You are on the left of the ship");
//	Crew.DeclareActionArea(1,_V(5,0,0),2.5,TRUE, "action_repaired.wav","You are on the right of the ship");
//	Crew.DeclareActionArea(2,_V(0,0,5),2.5,TRUE, "action_activated.wav","You are in front of the ship");
//	Crew.DeclareActionArea(3,_V(0,0,-5),2.5,TRUE,"action_activated.wav","You are behind the ship");
//	iActionAreaDemoStep=0;	// this is just to show a feature of action area, see below "DetectActionAreaActivated"

	//--------------------------------------------------------------
	// AddCrewMember
	// We'll add four default members for when the ship is spawned by Orbiter's scenario editor. 
	// parameters are name, age, cardiac pulse,weight (in kilogramm) and Function (Misc ID)
	// max four characters wich define the spacesuit used. (see UmmuMiscID in UMuSDK.h header)
	// Return 1 on success -1 on error (probably vessel full, name>25 char 
	// age, pulse or weight out of realistic range or MiscID>4 char)
	Crew.AddCrewMember("Jim Bob Sherwood",65,65,74,"Capt");		//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Alex Russo",19,67,55,"Pax");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Justin Russo",20,70,65,"Pax");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Max Russo",16,70,60,"Pax");		//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Harper Finkle",19,67,55,"Pax");		//(for name and id a-z A-Z 0-9 characters only)
	SelectedUmmuMember =0;  // our current selected member


	// The HUD display method variables, see PDF doc
	cUmmuHudDisplay[0] =0;	// Initialisation of UMmu hud char variable
	dHudMessageDelay =0;	// Initialisation of UMmu delay variable
	strcpy(SendHudMessage(),"Welcome aboard ! E=EVA 1,2=select UMmu A=Open/Close airlock S=info M=add crew");

	// The Add mmu without scenery editor variable see PDF doc
	cAddUMmuToVessel[0]=0;
}



// --------------------------------------------------------------
// Set vessel class caps 
// this is a custom clbkSetClassCaps
// called form the real one in ShuttlePB.cpp this to keep
// only UCGO specific code in this one.
// --------------------------------------------------------------
void JELAIR2011HotAirBalloon::clbkSetClassCapsUCgo (void)
{
	//UCGO 2.0 Initialisation, cargo slot pos, rot declaration
	hUcgo.Init(GetHandle());
	// declare up to 40 cargo slots
	hUcgo.DeclareCargoSlot(0,_V(0,-15,2),_V(0,0,0)); // slot 0 //front
	hUcgo.DeclareCargoSlot(1,_V(0,-15,-2),_V(0,0,0)); // slot 1 //back
	hUcgo.DeclareCargoSlot(2,_V(-2,-15,0),_V(0,0,80)); // slot 2 //left
	hUcgo.DeclareCargoSlot(3,_V(2,-15,0),_V(0,0,-80)); // slot 3 //right
	//these cargos hang outside gondola on each side

	// UCGO 2.0 Parameters settings
	hUcgo.SetReleaseSpeedInSpace(0.10f);	   // release speed of cargo in space in m/s
	hUcgo.SetMaxCargoMassAcceptable(550.0);   // max cargo mass in kg that your vessel can carry
	hUcgo.SetGrappleDistance(50);		       // grapple distance radius in meter from center of ship
	hUcgo.SetGlobalGroundReleasePos(_V(0,-15,3));// global release pos on ground

	// UCGO Variables initialisation
	cCargoHudDisplay[0]=0;						// Cargo hud display char variable
	dCargHudMessageDelay=0;						// Cargo hud display delay
	iSelectedCargo=-1;							// for the selection of cargos (-1 mean "default" see header)
	// welcome message with keys for users
	strcpy(SendCargHudMessage(),"Cargo key: C/SHF+C = grapple/release, 9/SHF+9 = snc editor add cargo, 8=infos on cargos");
}




// ==============================================================
// Airfoil lift/drag functions
// ==============================================================

void JELAIR2011HotAirBalloon::vlift (VESSEL *v, double aoa, double M, double Re,
	void *context, double *cl, double *cm, double *cd)
{
	static const double clp[] = {  // lift coefficient from -pi to pi in 90deg steps
		0.0,0.1,0.0,0.15,0.0,0.1 //2nd value is climb, 4th value is descend
	};
	static const double aoa_step = 90.0*RAD;
	double a, fidx;
	a = modf((aoa+PI)/aoa_step, &fidx);
	int idx = (int)(fidx+0.5);
	*cl = clp[idx]*(1.0-a) + clp[idx+1]*a;     // linear interpolation
	*cm = 0.0;
	*cd = 0.03;                // profile drag
	*cd += oapiGetInducedDrag (*cl, 0.07253716521192143413086030604176, 1.0); // induced drag
	*cd += oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);  // wave drag
}

void JELAIR2011HotAirBalloon::hlift (VESSEL *v, double aoa, double M, double Re,
	void *context, double *cl, double *cm, double *cd)
{
	static const double clp[] = {  // lift coefficient from -pi to pi in 90deg steps
		0.1,0.1,0.1,0.1,0.1,0.1 //must all be the same if the balloon is a sphere with no difference between front and side, cigar-shapes will have different values
	};
	static const double aoa_step = 90.0*RAD;
	double a, fidx;
	a = modf((aoa+PI)/aoa_step, &fidx);
	int idx = (int)(fidx+0.5);
	*cl = clp[idx]*(1.0-a) + clp[idx+1]*a;     // linear interpolation
	*cm = 0.0;
	*cd = 0.03;
	*cd += oapiGetInducedDrag (*cl, 0.07253716521192143413086030604176, 1.0); // induced drag
	*cd += oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);  // wave drag
}

// ==============================================================
// API callback interface
// ==============================================================







///////////////////////////////////////////////////////////////////////////////////////////
// clbkLoadState function of Orbiter - Save scenario when user exit Orbiter !REQUIRED!
// -------------
// Here we'll save all our crew members
///////////////////////////////////////////////////////////////////////////////////////////
void JELAIR2011HotAirBalloon::clbkSaveState(FILEHANDLE scn)
{

	// ORBITER, default vessel parameters
	SaveDefaultState (scn);

	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios(scn);

	// Save UCGO 2.0 cargo in scenario
	hUcgo.SaveCargoToScenario(scn);	
}



///////////////////////////////////////////////////////////////////////////////////////////
// clbkLoadStateEx function of Orbiter - Load Orbiter scenario !REQUIRED!
// ---------------
// Here we'll reload all crew member of our ship.
// notice the function will ERASE all defaults member set in clbkSetClassCaps.
// If you really want to add default crew add them also in this fonction after loading
// (when a ship is spawned by editor anyway this fonction below is never called)
///////////////////////////////////////////////////////////////////////////////////////////
void JELAIR2011HotAirBalloon::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;

		// Load UCGO 2.0 cargo from scenario
		if(hUcgo.LoadCargoFromScenario(line)==TRUE) // UCGO load cargo 
			continue;	

		// ORBITER, unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, status);
	}

	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu airlock state and set it accordingly
	// to your airlock. IE: call here "SetAirlockDoorState" TRUE of FALSE depend
	// of your animated airlock
}








// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new JELAIR2011HotAirBalloon (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (JELAIR2011HotAirBalloon*)vessel;
}





// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================
void JELAIR2011HotAirBalloon::clbkPostCreation (void)
{
	////////////////////////////////////////////////////////////////
	// 3-ORBITERSOUND EXAMPLE - INIT AND LOADING OF WAV
	// THIS MUST BE CALLED ABSOLUTELY IN THE "POSTCREATION CALLBACK" 
	////////////////////////////////////////////////////////////////
	// here we connect to OrbiterSound and store the returned ID in your class
	// this is the first thing to do. You must call this in "clbkPostCreation" 
	// (new version of ovcPostCreation wich is now obsolet)
	JELAIR_HAB20110208=ConnectToOrbiterSoundDLL3(GetHandle());

	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbienceLoop.wav",		REPLACE_AIR_CONDITIONNING);
	//get gondola open-air sound rather than aircondition

	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience1.wav",		REPLACE_COCKPIT_AMBIENCE_1);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience2.wav",		REPLACE_COCKPIT_AMBIENCE_2);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience3.wav",		REPLACE_COCKPIT_AMBIENCE_3);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience4.wav",		REPLACE_COCKPIT_AMBIENCE_4);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience5.wav",		REPLACE_COCKPIT_AMBIENCE_5);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience6.wav",		REPLACE_COCKPIT_AMBIENCE_6);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience7.wav",		REPLACE_COCKPIT_AMBIENCE_7);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience8.wav",		REPLACE_COCKPIT_AMBIENCE_8);
	ReplaceStockSound3(JELAIR_HAB20110208,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonAmbience9.wav",		REPLACE_COCKPIT_AMBIENCE_9);

	RequestLoadVesselWave3(JELAIR_HAB20110208,BURNER,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonGasBurner.wav",RADIO_SOUND);
	//gas burner sound
	RequestLoadVesselWave3(JELAIR_HAB20110208,VentFlap,"Sound\\JELAIR\\HAB20110208\\HotAirBalloonVentFlap.wav",RADIO_SOUND);
	//envelope flap to exhaust hot air

	SoundOptionOnOff3(JELAIR_HAB20110208,PLAYRADIOATC,FALSE);
	// we disable also the stock countdown when we take-off.. it's annoying :)
	SoundOptionOnOff3(JELAIR_HAB20110208,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

	SoundOptionOnOff3(JELAIR_HAB20110208,PLAYGPWS,FALSE);
	SoundOptionOnOff3(JELAIR_HAB20110208,PLAYDOCKLANDCLEARANCE,FALSE);

	//SetRadioFrequency3("296.800");		// Set the frequency 296.800. radio will play the sound contained in this folder
}




///////////////////////////////////////////////////////////////////////////////////////////
// USING HUD INTERFACE FOR FEEDBACK DISPLAY
//
// This method is explained in the PDF tutorial, now if you want to use another method
// comment the "UMMU" display lines in clbkHudDraw above and use the char variable 
// "cUmmuHudDisplay" wich will contain all the messages of UMMU to display it on VC or panel. 
// No other change required in code.
///////////////////////////////////////////////////////////////////////////////////////////
char *JELAIR2011HotAirBalloon::SendHudMessage()
{
	dHudMessageDelay=15;
	return cUmmuHudDisplay;
}





////////////////////////////////////////////////////////
// SendCargHudMessage
////////////////////////////////////////////////////////
char *JELAIR2011HotAirBalloon::SendCargHudMessage(void)
{
	dCargHudMessageDelay=15; // 15 seconds display delay for msg
	return cCargoHudDisplay;
}






bool JELAIR2011HotAirBalloon::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	VESSEL *pV = oapiGetFocusInterface();

	char buffer[256];
    memset(buffer, 0, sizeof(buffer));
	int len;

	VECTOR3 HorizonAirspeedVector;
	pV->GetHorizonAirspeedVector(HorizonAirspeedVector);
	double vsi_fpm;

	if ( pV->GroundContact() == true )
	{	
		vsi_fpm = 0;
	}
	else
	{	
		vsi_fpm = HorizonAirspeedVector.y * 196.85039370078738;
	}

	double current_radar_altitude_feet = pV->GetAltitude() * 3.280839895013123;
	double air_temperature_outside_the_envelope = pV->GetAtmTemperature();

	oapi::Font *font1;
	font1 = oapiCreateFont(hps->W/40, true, "Arial", FONT_NORMAL, 0);

	oapi::Font *font1small;
	font1small = oapiCreateFont(hps->W/70, true, "Arial", FONT_NORMAL, 0);

	skp->SetTextColor (0x0000FF);
	skp->SetTextAlign (oapi::Sketchpad::CENTER, oapi::Sketchpad::BASELINE);

	skp->SetFont (font1);
	len = sprintf_s(buffer, "VS %.0f FPM", vsi_fpm); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*3, buffer, len); //print readout from buffer

	skp->SetFont (font1);
	len = sprintf_s(buffer, "ENV %.0f C (%.0f F)", air_temperature_inside_the_envelope - 273.15, air_temperature_inside_the_envelope * 9 / 5 - 459.67 ); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*4, buffer, len); //print readout from buffer

	skp->SetFont (font1);
	len = sprintf_s(buffer, "ALT %.0f FT", current_radar_altitude_feet); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*5, buffer, len); //print readout from buffer


//	skp->SetFont (font1small);
//	len = sprintf_s(buffer, "Burner %d : AP %d : Vent %d", Control, ControlAutopilot, EnvelopeAirVentControl); //prepare readout in buffer
//	skp->Text(hps->W/10*9, hps->H/20*3, buffer, len); //print readout from buffer

	skp->SetFont (font1small);
	len = sprintf_s(buffer, "DIFF %.0f C (%.0f F)", (air_temperature_inside_the_envelope - 273.15) - (air_temperature_outside_the_envelope - 273.15), (air_temperature_inside_the_envelope * 9 / 5 - 459.67) - (air_temperature_outside_the_envelope * 9 / 5 - 459.67) ); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*7, buffer, len); //print readout from buffer

	skp->SetFont (font1small);
	len = sprintf_s(buffer, "AMB %.0f C (%.0f F)", air_temperature_outside_the_envelope - 273.15, air_temperature_outside_the_envelope * 9 / 5 - 459.67 ); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*8, buffer, len); //print readout from buffer


	double TotalMass = pV->GetMass(); //kg
	OBJHANDLE CurrentAtmosphere = pV->GetAtmRef(); //Get handle for the current surrounding atmosphere
	const ATMCONST *atmConst; //get atmosphere properties
	atmConst = oapiGetPlanetAtmConstants(CurrentAtmosphere);
	double SpecificGasConstant = atmConst->R; //specific gas constant [J/(K kg)], 286.91 on earth
	double AIRpressure = pV->GetAtmPressure(); //pascal (101300 standard)
	double AIRtemperature = pV->GetAtmTemperature();
	double envelope_temperature_needed_for_buoyancy = 0; //JELAIR, kg
	envelope_temperature_needed_for_buoyancy = 1 / ((1 / AIRtemperature) - (TotalMass / (envelope_volume * (AIRpressure / SpecificGasConstant)))); //363

	skp->SetFont (font1small);
	len = sprintf_s(buffer, "BUOYANCY %.0f C (%.0f F)", envelope_temperature_needed_for_buoyancy - 273.15, envelope_temperature_needed_for_buoyancy * 9 / 5 - 459.67 ); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*9, buffer, len); //print readout from buffer

	skp->SetFont (font1small);
	len = sprintf_s(buffer, "MASS %.0f KG (%.0f LBS)", pV->GetMass(), pV->GetMass() * 2.204622621848776 ); //prepare readout in buffer
	skp->Text(hps->W/10*9, hps->H/20*10, buffer, len); //print readout from buffer





	skp->SetFont (font1);
	if ( Control == 1  )
	{
		skp->Text (hps->W/10*4, hps->H/20*19, "Burner", 6);
	}
	if ( EnvelopeAirVentControl == 1  )
	{
		skp->Text (hps->W/10*5, hps->H/20*19, "Vent", 4);
	}
	if ( ControlAutopilot == 1  )
	{
		skp->Text (hps->W/10*6, hps->H/20*19, "Otto", 4);
	}



	skp->SetFont (font1);
	// UMmu display messages
	if(dHudMessageDelay>0)
	{
//		TextOut (hDC,5,hps->H/60*15,cUmmuHudDisplay,strlen(cUmmuHudDisplay));
		skp->Text(hps->W/10*5, hps->H/20*16, cUmmuHudDisplay, strlen(cUmmuHudDisplay)); //print readout from buffer
		dHudMessageDelay-=oapiGetSimStep();
		if(dHudMessageDelay<0)
			dHudMessageDelay=0;
	}

	// UCGO display messages
	if(dCargHudMessageDelay>0)
	{
		skp->Text(hps->W/10*5,hps->H/20*17,cCargoHudDisplay,strlen(cCargoHudDisplay));
		dCargHudMessageDelay-=oapiGetSimStep();
		if(dCargHudMessageDelay<0)
			dCargHudMessageDelay=0;
	}





	oapiReleaseFont(font1);
	oapiReleaseFont(font1small);




//	skp->Text (50, 50, "60", 2);
//
//	oapi::Pen *pen1;
//	pen1 = oapiCreatePen(1,3,0x0033FFFF); // create Yellow Pen
//	skp->SetPen(pen1);
//
//	skp->Line(10,10,50,50);  // draw a line
//
//	oapiReleasePen(pen1);                       // release Yellow pen
	return true;
}





// --------------------------------------------------------------
// clbkVisualCreated
// --------------------------------------------------------------
void JELAIR2011HotAirBalloon::clbkVisualCreated (VISHANDLE vis, int refcount)
{
	hUcgo.SetUcgoVisual(vis);	// must be called in clbkVisualCreated.
}






// --------------------------------------------------------------
// Process buffered key events
// --------------------------------------------------------------
int JELAIR2011HotAirBalloon::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	VESSEL *pV = oapiGetFocusInterface();

	if (!down) return 0; // only process keydown events
	if (Playback()) return 0; // don't allow manual user input during a playback

	if (KEYMOD_SHIFT (kstate))
	{
		switch (key) {
		case OAPI_KEY_UP:
	  		air_temperature_inside_the_envelope = air_temperature_inside_the_envelope + 10;
			return 1;
		}
	}
	else if (KEYMOD_CONTROL (kstate))
	{
		switch (key) {
		case OAPI_KEY_SPACE:
	  		ControlAutopilot = ControlAutopilot + 1;
			return 1;
		}
	}
	else
	{
		switch (key) {
		case OAPI_KEY_SPACE:
	  		Control = Control + 1;
			return 1;
		}
		switch (key) {
		case OAPI_KEY_ESCAPE:
	  		EnvelopeAirVentControl = EnvelopeAirVentControl + 1;
			return 1;
		}
	}
//	return 0;


	//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	//
	// ADD REALISM: It's your responsabilities also to set ship's control accordingly to crew aboard.
	//				If you want to disable control if no one is aboard have a look at "SetADCtrlMode()"
	//				and "SetAttitudeMode()" functions of Orbiter. To disable thrusters set their fuel 
	//				ressource to NULL.
	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		// we provide feedback to user (You can display a message on panel or wathewer)
		// here below all the return code possible:
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendHudMessage(),"Transfer to docked ship Ok - %s transfered",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendHudMessage(),"EVA OK - %s left the gondola",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendHudMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendHudMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendHudMessage(),"Error, no person by this name in gondola");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendHudMessage(),"Error, docked ship do not use UMmu 2.0, ask author to add it");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendHudMessage(),"Misc error with UMMU install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard, unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer"
			", please press \"E\" to EVA",SelectedUmmuMember,
			Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,
			Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard, unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA",
		SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),
		Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the virtual UMMU airlock door on/off
	if(key==OAPI_KEY_A&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// switch state
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
		// display state
		if(Crew.GetAirlockDoorState()==TRUE)
			strcpy(SendHudMessage(),"Airlock is now open");	
		else
			strcpy(SendHudMessage(),"Airlock is now closed");	
		return 1;
	}
	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendHudMessage(),"%i souls in gondola %s, with room for %i more",
			Crew.GetCrewTotalNumber(),GetName(),5-Crew.GetCrewTotalNumber());
		return 1;
	}

	//---------------------------------------------------------------------------
	// ADD some Fun, Eject the guy, No check of all return code here to keep listing small and clear.
	// Notice eject function doesn't check airlock state at all.
	// GOOD IDEA: Get the Object's handle after ejection with function "GetObjHandleOfLastEVACrew"
	// and add to it one very small tank, thruster and smoke, then fire thruster (see pilot ejection of DGIV)
	// BAD IDEA: Not testing the handle returned by "GetObjHandleOfLastEVACrew" before using may
	// cause a CTD if by any bad luck the pointer is invalid (handle==NULL)
	if(key==OAPI_KEY_P&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(Crew.EjectCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember))==EVA_OK)
			sprintf(SendHudMessage(),"%s JUMPED OUT OF GONDOLA",Crew.GetLastEvaedCrewName());
		SelectedUmmuMember=0;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "L" then EVA someone)
	// better idea is to use the new UMMU Id definition
	// look readme.txt in folder "config/UMMUIdConfig"
	if(key==OAPI_KEY_L&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendHudMessage(),"Mesh changed");
		return 1;
	}

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	if(key==OAPI_KEY_M&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		AddUMmuToVessel(TRUE);
	}


	// 9 key "select" one cargo on disk (cycle)
	if(key==OAPI_KEY_9&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendCargHudMessage(),"%s - selected",hUcgo.ScnEditor_SelectNextCargoAvailableOnDisk());
		return 1;
	}
	// SHIFT+9 key "add last cargo selected by key 9"
	// If iSelectedCargo=-1 (default) add to the first free slot found
	if(key==OAPI_KEY_9&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(hUcgo.ScnEditor_AddLastSelectedCargoToSlot(iSelectedCargo)==TRUE)
		{
			strcpy(SendCargHudMessage(),"Cargo added to gondola");
		}
		else
		{
			if(iSelectedCargo<0)
			{
				strcpy(SendCargHudMessage(),"Cargo not added (gondola full, weight excess ?)");
			}
			else
			{
				strcpy(SendCargHudMessage(),"Cargo not added (slot full, gondola full, weight excess ?)");
			}
		}
		return 1;
	}
	// "C" grapple cargo. If iSelectedCargo=-1 (default) add to the first free slot found
	if(key==OAPI_KEY_C&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		int ReturnedCode=hUcgo.GrappleOneCargo(iSelectedCargo);
		// for return code list see function "GrappleOneCargo" in the header
		switch(ReturnedCode)
		{
		case 1:
			strcpy(SendCargHudMessage(),"Cargo grappled");
			break;
		case 0:
			strcpy(SendCargHudMessage(),"No cargo in range");
			break;
		case -1:
			strcpy(SendCargHudMessage(),"Cargo exceed maximum allowed mass");
			break;
		case -2:
			strcpy(SendCargHudMessage(),"Bad config, mesh not found or slot not declared");
			break;
		case -3:
			strcpy(SendCargHudMessage(),"Can't grapple cargo, slot not empty");
			break;
		case -4:
			strcpy(SendCargHudMessage(),"Can't grapple cargo, doors closed ");
			break;
		case -5:
			strcpy(SendCargHudMessage(),"Can't grapple cargo, gondola full");
			break;
		default:
			strcpy(SendCargHudMessage(),"Misc error Unable to grapple cargo");
		}
		return 1;
	}
	// SHIFT+C release cargo. If iSelectedCargo=-1 (default) release the first free slot found
	if(key==OAPI_KEY_C&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(hUcgo.ReleaseOneCargo(iSelectedCargo)!=FALSE)
		{
			strcpy(SendCargHudMessage(),"Cargo released");
		}
		else
		{
			strcpy(SendCargHudMessage(),"Cargo not released (empty slot, no cargo aboard?)");
		}
		return 1;
	}
	// "8" show some info on cargo
	if(key==OAPI_KEY_8&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendCargHudMessage(),"%i cargos aboard. "
		"Total cargos weight: %.0fkg",hUcgo.GetNbrCargoLoaded(),hUcgo.GetCargoTotalMass());
		return 1;
	}



	return 0;
}



//-------------------------------------------------------------------------
// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
// without scenery editor"
bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr=(char*)data;	if(*cPtr==2){*cPtr=3;strcpy(cPtr+2,str);}
	else if(*cPtr==4){*cPtr=5;strcpy(cPtr+42,str);}
	else if(*cPtr==6){*cPtr=7;strcpy(cPtr+82,str);}return true;
}
void JELAIR2011HotAirBalloon::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding==FALSE&&cAddUMmuToVessel[0]==0)
		return;
	if(bStartAdding==TRUE){
		int salut=sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel,0,sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0]=1;
	}
	else if(cAddUMmuToVessel[0]==1){
		cAddUMmuToVessel[0]=2;
		oapiOpenInputBox ("Enter new person's name (or escape)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==3){
		cAddUMmuToVessel[0]=4;
		oapiOpenInputBox ("Enter person's age",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==5){
		cAddUMmuToVessel[0]=6;
		oapiOpenInputBox ("Enter function (Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==7){
		cAddUMmuToVessel[0]=0;
		int Age=max(5,min(100,atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2],Age,70,70,&cAddUMmuToVessel[82])==TRUE){
			sprintf(SendHudMessage(),"Person \"%s\" aged %i added to gondola",&cAddUMmuToVessel[2],Age);
		}
		else{
			strcpy(SendHudMessage(),"ERROR: Person not added (gondola full?)");
		}
	}
}
// END of  "Allow user to add crew to your ship without scenery editor"
//-------------------------------------------------------------------------









// --------------------------------------------------------------
// Frame update
// --------------------------------------------------------------
void JELAIR2011HotAirBalloon::clbkPreStep (double simt, double simdt, double mjd)
{
	VESSEL *pV = oapiGetFocusInterface();
	oapiSetTimeAcceleration(1);

//	VECTOR3 ShiftMeshes = {0, 2, 0};
//	pV->ShiftMeshes(ShiftMeshes);
//	int MeshOffset;
//	VECTOR3 MeshVector = {0, 0.02, 0};
//	pV->ShiftMesh(0, MeshVector);
//	pV->GetMeshOffset(0, MeshVector);
//	sprintf (oapiDebugString(), "X %f, Y %f, Z %f", MeshVector.x, MeshVector.y, MeshVector.z);

	OBJHANDLE CurrentAtmosphere = pV->GetAtmRef(); //Get handle for the current surrounding atmosphere
	const ATMCONST *atmConst; //get atmosphere properties
	atmConst = oapiGetPlanetAtmConstants(CurrentAtmosphere);
//	double gamma = atmConst->gamma; // Retrieve the ratio of specific heats
	double SpecificGasConstant = atmConst->R; //specific gas constant [J/(K kg)], 286.91 on earth


//	double TASkts = pV->GetAirspeed()*1.94384449;
//	double ALTft = pV->GetAltitude()*3.2808399;
//	int AutopilotAltitudeHold;

//	int is_envelope_overheated_and_melted;

	double WindSpeed_meter_per_second;
	if ( pV->GetAltitude() < 20000 )
	{
		WindSpeed_meter_per_second = pV->GetAltitude()*0.0075;

		if ( pV->GetAltitude() > 12000 )
		{
			WindSpeed_meter_per_second = WindSpeed_meter_per_second + ((pV->GetAltitude() - 12000) * -0.015);
		}
	}
	else
	{
		WindSpeed_meter_per_second = 0;
	}
	//simple wind-speed simulator with rising winds up to 12000 meter and then falling again up to 20000 m. Currently no winds above 20000 m
		if ( pV->GetAltitude() < 1000 )
		{
			pV->AddForce(_V(WindSpeed_meter_per_second * -75, 0, WindSpeed_meter_per_second * 100), _V(0,0,0));
		}
		else if ( pV->GetAltitude() < 2000 )
		{
			pV->AddForce(_V(WindSpeed_meter_per_second * 100, 0, WindSpeed_meter_per_second * -100), _V(0,0,0));
		}
		else if ( pV->GetAltitude() < 3000 )
		{
			pV->AddForce(_V(WindSpeed_meter_per_second * 100, 0, WindSpeed_meter_per_second * 100), _V(0,0,0));
		}
		else if ( pV->GetAltitude() < 4000 )
		{
			pV->AddForce(_V(WindSpeed_meter_per_second * -100, 0, WindSpeed_meter_per_second * -100), _V(0,0,0));
		}
		else if ( pV->GetAltitude() < 5000 )
		{
			pV->AddForce(_V(WindSpeed_meter_per_second * -100, 0, WindSpeed_meter_per_second * 100), _V(0,0,0));
		}
		else
		{
			pV->AddForce(_V(WindSpeed_meter_per_second * 100, 0, WindSpeed_meter_per_second * 100), _V(0,0,0));
		}
		//simple wind-simulation with different wind-directions depending on altitude

	

	double AIRdensity = pV->GetAtmDensity();
	double AIRpressure = pV->GetAtmPressure(); //pascal (101300 standard)
	double AIRtemperature = pV->GetAtmTemperature();
	double TotalMass = pV->GetMass(); //kg
	double Pitch = pV->GetPitch() * 3.1415926535897932384626433832795 / 180;
	double Bank = pV->GetBank() * 3.1415926535897932384626433832795 / 180;

	double envelope_temperature_needed_for_buoyancy = 0; //JELAIR, kg
	double envelope_air_density = AIRpressure / (SpecificGasConstant * air_temperature_inside_the_envelope); //can be used to calculate lift in kg
    double AIRdensityDifferenceFactor = AIRdensity / envelope_air_density; //multiply with air-volume to get lift in kg

	OBJHANDLE FuelTankHandle = pV->GetPropellantHandleByIndex(0); //Get handle for fuel-tank
	double TotalFuelMass = pV->GetPropellantMass(FuelTankHandle); //kg
	double FuelBurnRate = 0.001;

	VECTOR3 HorizonAirspeedVector;
	pV->GetHorizonAirspeedVector(HorizonAirspeedVector);

	VECTOR3 fvec;
	pV->GetForceVector(fvec);

	VECTOR3 Gvec;
	pV->GetWeightVector(Gvec);
	//JELinfo: Gvec.y / TotalMass = gravity (9.8 at earth)
	//sprintf (oapiDebugString(), "GvecX %.2f, GvecY %.2f, GvecZ %.2f, fvecX %.2f, fvecY %.2f, fvecZ %.2f", Gvec.x, -(Gvec.y / TotalMass), Gvec.z, fvec.x, fvec.y / TotalMass, fvec.z);



	//	lift_generated_by_the_envelope = envelope_volume * (AIRpressure / 2.87) * ( (1 / AIRtemperature) - (1 / air_temperature_inside_the_envelope) ); //kg
//	envelope_temperature_needed_for_buoyancy = TotalMass / (envelope_volume * (AIRpressure / 2.87)) / (1 / AIRtemperature) * 273.15;
	envelope_temperature_needed_for_buoyancy = 1 / ((1 / AIRtemperature) - (TotalMass / (envelope_volume * (AIRpressure / SpecificGasConstant)))); //363
	//TotalMass = 1,000 kg
	//envelope_volume * (AIRpressure / 2.87) = 1,000,000
	//1 / AIRtemperature (288k aka 15c) = 0.00347
	//mass (723 kg) / 999,482 = 0,00072 / 0.00347 = 0.2075 * 273 = 56.6475
	//1 / 273k aka 0c = 0.00366
	//0.00072 / 0.00347 = 0.2075 * 273 = 56.6475
	//0.00072 / 0.00366 = 0.1967 * 273 = 53.6991
	//90c = 0.3297 (when using 1 / temp factor)

	//	lift_generated_by_the_envelope = 2831.7 * (1013 / 2.87) * ( (1 / 288) - (1 / 363) ); //kg
	//                                            (352.96167  )     (0.00347)   (0.00275)
	//                                   (999,482             )   (0.00072                )
	//                                   (720                                             )

	//	                           720 / 2831.7 * (1013 / 2.87) * ( (1 / 288) - (1 / 363) ); //kg
	//                                            (352.96167  )     (0.00347)   (0.00275)
	//                                   (999,482             )   (0.00072                )
	//                             (0.00072                   )   (0.00072                )
	//                                   (363 or 0.00275 or 0.00072                       )



	/*
	int Control = 0; //JELAIR, is the burner ON or OFF
	double F = 0; //JELAIR, amount of Newton force added (this is how the balloon flies)

	double lift_generated_by_the_envelope = 0; //JELAIR, kg
	double envelope_volume = 0; //JELAIR, m^3 (cubic meter)
	double ambient_air_pressure_outside_envelope = 0; //JELAIR, the ambient air pressure of the surrounding air
	double air_temperature_outside_the_envelope = 0; //JELAIR, the ambient air temperature of the surrounding air
	double air_temperature_inside_the_envelope = 0; //JELAIR, the air temperature of the air inside the envelope

	double air_density_inside_the_envelope = 0; //JELAIR
	double air_mass_inside_the_envelope = 0; //JELAIR
	*/


	if ( ControlAutopilot == 0 ) //ControlAutopilot OFF
	{
		AutopilotAltitudeHold = int (pV->GetAltitude());
	}
	if ( ControlAutopilot == 1 ) //ControlAutopilot ON
	{
		if ( int (pV->GetAltitude()) + 10 < AutopilotAltitudeHold )
		{
			if ( air_temperature_inside_the_envelope - 3 < envelope_temperature_needed_for_buoyancy && air_temperature_inside_the_envelope < 393.15 )
			{
				Control = 1;
			}
			else
			{
				Control = 2;
			}
		}
		else if ( int (pV->GetAltitude()) - 10 > AutopilotAltitudeHold )
		{
			if ( air_temperature_inside_the_envelope + 3 < envelope_temperature_needed_for_buoyancy && air_temperature_inside_the_envelope < 393.15 )
			{
				Control = 1;
			}
			else
			{
				Control = 2;
			}
		}
		else
		{
			if ( air_temperature_inside_the_envelope < envelope_temperature_needed_for_buoyancy )
			{
				Control = 1;
			}
			else
			{
				Control = 2;
			}
		}
	}
	if ( ControlAutopilot > 1 ) //loop back
	{
		ControlAutopilot = 0;
	}
	//ControlAutopilot program

	if ( TotalFuelMass <= 0 ) //check if there is any fuel
	{
		Control = 2;
	}
	//if no more fuel then disallow burn

	if ( Control == 1 ) //burner active
	{
		air_temperature_inside_the_envelope = air_temperature_inside_the_envelope + 0.005;

		//air_temperature_inside_the_envelope, MAX temp = 120 celsius (393.15 K), MELTING temp = 230 celsius (503.15 K)
		if ( air_temperature_inside_the_envelope > 510 )
		{
			is_envelope_overheated_and_melted = 1;
			PB_CS.x = 2;
			PB_CS.y = 2;
			PB_CS.z = 2;
			pV->SetCrossSections (PB_CS);
		}

		TotalFuelMass = TotalFuelMass - FuelBurnRate; TotalFuelMass > 0 ? TotalFuelMass : 0;
		pV->SetPropellantMass(FuelTankHandle, TotalFuelMass);
		//5>3 ? a : b      // returns the value of a, since 5 is greater than 3.
		//a>b ? a : b      // returns whichever is greater, a or b. 

		if ( ControlAutopilot != 1 ) //ControlAutopilot OFF
		{
			PlayVesselRadioExclusiveWave3(JELAIR_HAB20110208,BURNER,255);	// 255=volume maximum 
		}
	}
	if ( Control == 2 ) //burner inactive
	{
		air_temperature_inside_the_envelope = air_temperature_inside_the_envelope - 0.00135;
		StopVesselWave3(JELAIR_HAB20110208,BURNER);
	}
	if ( Control > 2 ) //loop back
	{
		Control = 1;
		StopVesselWave3(JELAIR_HAB20110208,BURNER);
	}
	//burner program

	if ( EnvelopeAirVentControl == 0 ) //loop back
	{
		StopVesselWave3(JELAIR_HAB20110208,VentFlap);
	}
	if ( EnvelopeAirVentControl == 1 ) //envelope air-vent open
	{
		air_temperature_inside_the_envelope = air_temperature_inside_the_envelope - 0.01;
		PlayVesselRadioExclusiveWave3(JELAIR_HAB20110208,VentFlap,255);	// 255=volume maximum 
	}
	if ( EnvelopeAirVentControl > 1 ) //loop back
	{
		EnvelopeAirVentControl = 0;
		StopVesselWave3(JELAIR_HAB20110208,VentFlap);
	}
	//envelope air-vent program

	if ( air_temperature_inside_the_envelope < AIRtemperature )
	{
		air_temperature_inside_the_envelope = AIRtemperature;
		EnvelopeAirVentControl = 0;
	}
	//restrict envelope from being colder than outside ambient air-temperature

	if ( is_envelope_overheated_and_melted == 1 )
	{
		air_temperature_inside_the_envelope = AIRtemperature;
		EnvelopeAirVentControl = 0;
	}
	//if you overheat and melt the envelope you can nolonger heat it up

	//	lift_generated_by_the_envelope = envelope_volume * (AIRpressure / 2.87) * ( (1 / AIRtemperature) - (1 / air_temperature_inside_the_envelope) ); //kg
	lift_generated_by_the_envelope = envelope_volume * (AIRpressure / SpecificGasConstant) * ( (1 / AIRtemperature) - (1 / air_temperature_inside_the_envelope) ); //kg
	if ( Control == 0 )
	{
//		lift_generated_by_the_envelope = 0;
//		pV->AddForce(_V(0,0,0), _V(0,0,0));

		StopVesselWave3(JELAIR_HAB20110208,BURNER);
	}

	F = lift_generated_by_the_envelope * -(Gvec.y / TotalMass); //buoyancy force F in newton = lift in kg * gravity pull on vessel in m/s (the minus is sign reversal)
	//F = lift_generated_by_the_envelope * 9.81; //buoyancy force F in newton = lift in kg * gravity pull on vessel in m/s
	pV->AddForce(_V(0,F,0), _V(0,0,0));

	pV->AddForce(_V(0,0,Pitch * 1000), _V(0,300,0)); //keeps the balloon level with horizon
	pV->AddForce(_V(Bank * 1000,0,0), _V(0,300,0)); //keeps the balloon level with horizon


/*
	COLOUR4 col_d = {0.9,0.8,1,0};
	COLOUR4 col_s = {1.9,0.8,1,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	LightEmitter *le = AddPointLight (_V(0,0,-10), 200, 1e-3, 0, 2e-3, col_d, col_s, col_a);

	bool ControlStatusForBurnerLight;
		ControlStatusForBurnerLight = false;
	if (Control == 1)
	{
		ControlStatusForBurnerLight = true;
	}
	else
	{
		ControlStatusForBurnerLight = false;
	}

	le->Activate(ControlStatusForBurnerLight);
*/
	//attempt to have burner switch on and off a light-source, can only switch on at this point but not off again, more work needed with this

//	sprintf (oapiDebugString(), "Control %d, AP %.0d, Elift %.0f, Elift2 %.0f, Etemp %.1f, TotalMass %.2f, Edens %.2f, Atemp %.2f, buoyEtemp %.2f, APalt %.0d", Control, ControlAutopilot, lift_generated_by_the_envelope, AIRdensityDifferenceFactor * (envelope_volume * envelope_air_density) - (envelope_volume * envelope_air_density), air_temperature_inside_the_envelope - 273.15, TotalMass, envelope_air_density, AIRtemperature - 273.15, envelope_temperature_needed_for_buoyancy - 273.15, AutopilotAltitudeHold);
}















///////////////////////////////////////////////////////////////////////////////////////////
// clbkPostStep function of Orbiter - Called each frame !REQUIRED!
// ----------------------
///////////////////////////////////////////////////////////////////////////////////////////
void JELAIR2011HotAirBalloon::clbkPostStep (double simtt, double simdt, double mjd)
{

	//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	// Here the routine that detect if someone entered the ship (eva or transfer)
	// No need to manage anything here all is automatic, crew is added to your ship, ship's weight is updated,
	// and UMmu vessel is automatically deleted from Orbiter. You may just look the return
	// code to see if someone entered and display wathewer message on panel or other.
	// notice it's FPS friendly, function process only 4 time per second not each frame 
	// and return immediately if airlock closed or no Ummu is detected in vincinity. 
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf(SendHudMessage(),"%s \"%s\" aged %i was transfered to our gondola",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
		,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf(SendHudMessage(),"%s \"%s\" aged %i entered into our gondola",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
		Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}


/*
	//----------------------------------------------------------------------------
	// OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL
	// DetectActionAreaActivated
	// This function will detect one keypress of UMMU inside one action area (see "DeclareActionArea" above)
	// It can be used to open doors, activate things, repair system etc. etc. 
	int ActionAreaReturnCode=Crew.DetectActionAreaActivated();
	if(ActionAreaReturnCode>-1)
	{
		// this is just an example, we have four area declared
		// action area ID 0 triggered 
		if(ActionAreaReturnCode==0)
		{
			// do something cool here
		}
		// action area ID 1 triggered
		else if(ActionAreaReturnCode==1)
		{
			// As you can edit action area in real time: relocation, sound, text, state
			// the possiblity are endless. you can for example order a UMMU to go to an external panel
			// and open it, then go to engine exhaust and "repair" it then back to panel to close it etc.
			if(iActionAreaDemoStep==0)
			{
				Crew.SetActionAreaText(1,"You are still on the right but this is 2nd keypress");
				iActionAreaDemoStep++;
				// do something cool here
			}
			else if(iActionAreaDemoStep==1)
			{
				// remember you change action area's parameter for the NEXT keypress of UMMU
				Crew.SetActionAreaText(1,"You are still on the right but this is 3rd keypress");
				iActionAreaDemoStep++;
				// do something cool here
			}
			else
			{
				Crew.SetActionAreaText(1,"You are still on the right but this is last keypress");
				// and of course you can change sound, location, state, text, here some examples:
				//Crew.SetActionAreaPos(1,_V(5,0,5)); // change location
				//Crew.SetActionAreaWav(1,"MyAddon\PlayRepaired.wav"); // change sound
				//Crew.SetActionAreaWav(1,"MyAddon\DoorClosed.wav"); // change sound
				//Crew.SetActionAreaText(1,NULL); // sound or text are not mandatory, here we set no text.
				// etc. etc.
			}
		}
		// action area ID 2 triggered
		else if(ActionAreaReturnCode==2)
		{
			// do something cool here
		}
		// action area ID 3 triggered
		else if(ActionAreaReturnCode==3)
		{
			// do something cool here
		}
	}
*/

	//----------------------------------------------------------------------------
	// SetCrewMemberPulseBySlotNumber
	// Now we will kill all our crew aboard if we crash on ground. First we test
	// if we have ground contact, then we check vertical speed and if it's more than
	// -3 m/s we kill all people aboard. Of course in your code you'll do that only
	// one time using a bool flag if(!bCrewAlreadyKilled) for example. Here we do it each
	// time step for code ease of read.
	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-10)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			for(I=0;I<Crew.GetCrewTotalNumber();I++)
			{
				Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
			}
			strcpy(SendHudMessage(),"Crash - No survivors");
		}

		// TIPS: the vertical speed is often reset to zero when there is ground contact
		// this may bug somewhat the death of your crew.
		// to have an accurate VertSpeed at touchdown I recommand to record it at very END 
		// of timestep and use this "old" value. The next frame you'll have the vertspeed value 
		// of *last frame* just before the crash (GroundContact). This ensure an accurate 
		// verticalspeed value. (keep this value in your vessel class and don't forget to 
		// initialize it at zero in setclasscap)
	}

	//---------------------------------------------------------------------------
	// WarnUserUMMUNotInstalled - IMPORTANT helper
	// Put here this function and users will be automatically warned if they don't have
	// UMMU installed or if it's outdated. Warning text duration is 20 seconds and text is:
	// [AddonName] require "Universal MMU" ver 2.0 or higher. Download at www.orbiter.dansteph.com (message countdown in seconds)
	// an additonal "your version of UMMU is outdated" is displayed the last 5 seconds if they have version lower than 2.0
	Crew.WarnUserUMMUNotInstalled("JELAIR2011HotAirBalloon");

     // At the very end of clbkPostStep or clbkPreStep
     hUcgo.WarnUserUCGONotInstalled("JELAIR2011HotAirBalloon");

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	AddUMmuToVessel();
} 
